<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace EventcorrelationModule\Controller;


use App\Lib\Interfaces\HoststatusTableInterface;
use App\Lib\Interfaces\ServicestatusTableInterface;
use App\Model\Entity\Changelog;
use App\Model\Table\ChangelogsTable;
use App\Model\Table\ContactgroupsTable;
use App\Model\Table\ContactsTable;
use App\Model\Table\ContainersTable;
use App\Model\Table\HostsTable;
use App\Model\Table\HosttemplatesTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\TimeperiodsTable;
use App\Model\Table\WidgetsTable;
use Cake\Datasource\Exception\RecordNotFoundException;
use Cake\Http\Exception\BadRequestException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use EventcorrelationModule\Form\EventcorrelationForm;
use EventcorrelationModule\Model\Table\EventcorrelationSettingsTable;
use EventcorrelationModule\Model\Table\EventcorrelationsTable;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\Comparison\HostComparisonForSave;
use itnovum\openITCOCKPIT\Core\Comparison\ServiceComparisonForSave;
use itnovum\openITCOCKPIT\Core\HostConditions;
use itnovum\openITCOCKPIT\Core\Hoststatus;
use itnovum\openITCOCKPIT\Core\HoststatusFields;
use itnovum\openITCOCKPIT\Core\KeyValueStore;
use itnovum\openITCOCKPIT\Core\ServiceConditions;
use itnovum\openITCOCKPIT\Core\Servicestatus;
use itnovum\openITCOCKPIT\Core\ServicestatusFields;
use itnovum\openITCOCKPIT\Core\UUID;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use itnovum\openITCOCKPIT\Core\Views\ContainerPermissions;
use itnovum\openITCOCKPIT\Core\Views\EvcContainerPermissions;
use itnovum\openITCOCKPIT\Core\Views\Host;
use itnovum\openITCOCKPIT\Core\Views\Service;
use itnovum\openITCOCKPIT\Core\Views\UserTime;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use itnovum\openITCOCKPIT\Filter\HostFilter;
use itnovum\openITCOCKPIT\Filter\ServiceFilter;

/**
 * Class EventcorrelationsController
 * @package EventcorrelationModule\Controller
 */
class EventcorrelationsController extends AppController {

    public function index() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');

        $HostFilter = new HostFilter($this->request);
        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $HostFilter->getPage());

        $where = $HostFilter->indexFilter();
        $where['Hosts.host_type'] = EVK_HOST;
        $HostConditions = new HostConditions(
            $where
        );
        if ($this->hasRootPrivileges === false) {
            $HostConditions->setContainerIds($this->MY_RIGHTS);
        }

        $HostConditions->setIncludeDisabled(true);
        $HostConditions->setOrder($HostFilter->getOrderForPaginator('Hosts.name', 'asc'));

        $evcHosts = $HostsTable->getHostsByHostConditions($HostConditions, $PaginateOMat);
        foreach ($evcHosts as $index => $evcHost) {
            //ITC-1822
            $hasViewPermission = $this->hasRootPrivileges;
            $hasWritePermission = $this->hasRootPrivileges;

            if ($this->hasRootPrivileges === false) {
                $firstLayerHosts = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($evcHost['Host']['id']);
                $hostIds = array_unique(Hash::extract($firstLayerHosts, '{n}.service.host_id'));

                if (!empty($hostIds)) {
                    //EVC has own services defined
                    //Check each first layer service (through host) container permissions

                    $allContainerIdsInEvc = [];
                    foreach ($hostIds as $hostId) {
                        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
                        $allContainerIdsInEvc[$hostId] = $host->getContainerIds();
                    }

                    $EvcContainerPermissions = new EvcContainerPermissions(
                        $this->MY_RIGHTS_LEVEL,
                        $allContainerIdsInEvc,
                        $evcHost['Host']['container_id']
                    );
                    $hasWritePermission = $EvcContainerPermissions->hasEditPermission();
                    $hasViewPermission = $EvcContainerPermissions->hasViewPermission();

                } else {
                    //Empty EVC (no own services defined)
                    //Use EVC host container_id itself for permission check
                    $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$evcHost['Host']['container_id']]);
                    $hasWritePermission = $ContainerPermissions->hasPermission();
                    $hasViewPermission = $ContainerPermissions->hasViewPermission();
                }
            }

            $evcHosts[$index]['hasViewPermission'] = $hasViewPermission;
            $evcHosts[$index]['hasWritePermission'] = $hasWritePermission;
        }

        $this->set('all_evc_hosts', $evcHosts);
        $this->viewBuilder()->setOption('serialize', ['all_evc_hosts']);
    }

    public function add() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        if ($this->request->is('post')) {
            if (!$this->request->getData('Host.container_id') || !$this->request->getData('Host.hosttemplate_id')) {
                $errors = [];
                $this->response = $this->response->withStatus(400);
                if (!$this->request->getData('Host.container_id')) {

                    $errors['container_id'] = [
                        'empty' => __('This field cannot be left empty')
                    ];
                }
                if (!$this->request->getData('Host.hosttemplate_id')) {
                    $errors['hosttemplate_id'] = [
                        'empty' => __('This field cannot be left empty')
                    ];
                }
                $this->set('error', $errors);
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }


            /** @var $HosttemplatesTable HosttemplatesTable */
            $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');
            /** @var $HostsTable HostsTable */
            $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
            $hosttemplateId = $this->request->getData('Host.hosttemplate_id');
            if (!$HosttemplatesTable->existsById($hosttemplateId)) {
                throw new NotFoundException(__('Invalid host template'));
            }

            $requestData = $this->request->getData();
            $hosttemplate = $HosttemplatesTable->getHosttemplateForDiff($hosttemplateId);
            $HostComparisonForSave = new HostComparisonForSave($requestData, $hosttemplate, true);
            $hostData = $HostComparisonForSave->getDataForSaveForAllFields();
            $hostData['uuid'] = UUID::v4();
            $hostData['address'] = '127.1.2.3'; //V2 legacy
            $hostData['host_type'] = EVK_HOST;

            //set usage flag for every new evc host
            $hostData['usage_flag'] = EVENTCORRELATION_MODULE;

            //Add required fields for validation
            $hostData['hosttemplate_flap_detection_enabled'] = $hosttemplate['Hosttemplate']['flap_detection_enabled'];
            $hostData['hosttemplate_flap_detection_on_up'] = $hosttemplate['Hosttemplate']['flap_detection_on_up'];
            $hostData['hosttemplate_flap_detection_on_down'] = $hosttemplate['Hosttemplate']['flap_detection_on_down'];
            $hostData['hosttemplate_flap_detection_on_unreachable'] = $hosttemplate['Hosttemplate']['flap_detection_on_unreachable'];

            $host = $HostsTable->newEntity($hostData);

            $HostsTable->save($host);
            if ($host->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $host->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            } else {
                //No errors
                $User = new User($this->getUser());

                $extDataForChangelog = $HostsTable->resolveDataForChangelog($requestData);
                /** @var  ChangelogsTable $ChangelogsTable */
                $ChangelogsTable = TableRegistry::getTableLocator()->get('Changelogs');

                $changelog_data = $ChangelogsTable->parseDataForChangelog(
                    'add',
                    'hosts',
                    $host->get('id'),
                    OBJECT_HOST,
                    $host->get('container_id'),
                    $User->getId(),
                    $host->get('name'),
                    array_merge($requestData, $extDataForChangelog)
                );

                if ($changelog_data) {
                    /** @var Changelog $changelogEntry */
                    $changelogEntry = $ChangelogsTable->newEntity($changelog_data);
                    $ChangelogsTable->save($changelogEntry);
                }


                if ($this->isJsonRequest()) {
                    $this->serializeCake4Id($host); // REST API ID serialization
                    return;
                }
            }
            $this->set('host', $host);
            $this->viewBuilder()->setOption('serialize', ['host']);
        }
    }

    /**
     * @param int|null $id
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     */
    public function view($id = null, $isTableView = false) {
        if (!$this->isApiRequest() && $isTableView === false) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */
        $EventcorrelationSettingsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.EventcorrelationSettings');
        /** @var ContainersTable $ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();

        if (!$HostsTable->existsByIdAndType($id, EVK_HOST)) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        $currentEventcorrelationSettings = $EventcorrelationSettingsTable->getCurrentEventcorrelationSettings();
        $defaultEventcorrelationSettings = $EventcorrelationSettingsTable->getDefaultEventcorrelationSettings();

        $considerStateType = ($defaultEventcorrelationSettings['EVC_CONSIDER_STATETYPE']['value'] & $currentEventcorrelationSettings->get('configuration_option'));
        $showInfoForDisabledService = ($defaultEventcorrelationSettings['EVC_SHOW_INFO_FOR_DISABLED_SERVICES']['value'] & $currentEventcorrelationSettings->get('configuration_option'));
        $stateForDisabledService = $defaultEventcorrelationSettings['EVC_STATUS_DISABLED_SERVICE']['value'];
        if (!empty($currentEventcorrelationSettings)) {
            $stateForDisabledService = $currentEventcorrelationSettings->get('status_disabled_service');
        }
        $evcHosts = $EventcorrelationsTable->getEvcHostsById($id);

        $containerIds = $this->MY_RIGHTS;
        $hosts = [];

        if (!empty($evcHosts)) {
            $hosts = $HostsTable->getHostsByContainerId($containerIds, 'list', 'id', [
                'Hosts.disabled IN' => [1], //0 is defaut and will get merged so the query is IN(0,1) :facepalming:
                'Hosts.id IN'       => array_unique(Hash::extract($evcHosts, '{n}.service.host_id'))
            ]);
        }

        try {
            $evcTree = $EventcorrelationsTable->getEvcTreeData($id, array_keys($hosts));
            $rootElement = $EventcorrelationsTable->getRootelementByHostId($id);
        } catch (RecordNotFoundException $e) {
            $evcTree = [];
            $rootElement = null;
        }

        $hasWritePermission = $this->hasRootPrivileges;

        if ($this->hasRootPrivileges === false) {
            $firstLayerHosts = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($id);
            $hostIds = array_unique(Hash::extract($firstLayerHosts, '{n}.service.host_id'));

            if (!empty($hostIds)) {
                //EVC has own services defined
                //Check each first layer service (through host) container permissions

                $allContainerIdsInEvc = [];
                foreach ($hostIds as $hostId) {
                    $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
                    $allContainerIdsInEvc[$hostId] = $host->getContainerIds();
                }

                $EvcContainerPermissions = new EvcContainerPermissions(
                    $this->MY_RIGHTS_LEVEL,
                    $allContainerIdsInEvc,
                    $rootElement->get('host')->get('container_id')
                );
                $hasWritePermission = $EvcContainerPermissions->hasEditPermission();
                $hasViewPermission = $EvcContainerPermissions->hasViewPermission();

            } else {
                //Empty EVC (no own services defined)
                //Use EVC host container_id itself for permission check
                $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$rootElement->get('host')->get('container_id')]);
                $hasWritePermission = $ContainerPermissions->hasPermission();
            }
        }


        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        $serviceUuids = array_unique(Hash::extract($evcTree, '{n}.{*}.{n}.service.uuid'));

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields
            ->currentState()
            ->lastHardState()
            ->isHardstate()
            ->output()
            ->longOutput()
            ->lastStateChange()
            ->lastCheck()
            ->nextCheck()
            ->currentCheckAttempt()
            ->maxCheckAttempts()
            ->activeChecksEnabled();

        $servicestatus = $ServicestatusTable->byUuids($serviceUuids, $ServicestatusFields);
        $disabledServices = 0;
        foreach ($evcTree as $layerNumber => $layer) {
            foreach ($layer as $parentId => $childNodes) {
                foreach ($childNodes as $childIndex => $childNode) {
                    $Servicestatus = new Servicestatus([], $UserTime);
                    if (!empty($servicestatus[$childNode['service']['uuid']])) {
                        $Servicestatus = new Servicestatus(
                            $servicestatus[$childNode['service']['uuid']]['Servicestatus'],
                            $UserTime
                        );
                    }

                    if ($considerStateType && !$Servicestatus->isHardState()) {
                        $Servicestatus->setCurrentState($Servicestatus->getLastHardState());
                    }

                    if ($layerNumber == 0 && $childNode['service']['disabled']) {
                        $Servicestatus->setCurrentState($stateForDisabledService);
                        $Servicestatus->isHardState(true);
                        $Servicestatus->setOutput('DISABLED');
                        $disabledServices++;
                    }

                    $evcTree[$layerNumber][$parentId][$childIndex]['service']['servicestatus'] = $Servicestatus->toArray();
                }
            }
        }

        $this->set('rootElement', $rootElement);
        $this->set('evcTree', $evcTree);
        $this->set('stateForDisabledService', $stateForDisabledService);
        $this->set('showInfoForDisabledService', $showInfoForDisabledService);
        $this->set('disabledServices', $disabledServices);
        $this->set('hasWritePermission', $hasWritePermission);


        $this->viewBuilder()->setOption('serialize', [
            'evcTree',
            'rootElement',
            'stateForDisabledService',
            'showInfoForDisabledService',
            'disabledServices',
            'hasWritePermission'
        ]);
    }

    /**
     * @param int|null $id
     * @param int|null $highlightedHostId
     * @throws \Exception
     * @deprecated
     */
    public function editCorrelation($id = null, $highlightedHostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */
        $EventcorrelationSettingsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.EventcorrelationSettings');
        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();
        /** @var ServicetemplatesTable $ServicetemplatesTable */
        $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');

        //Event correlation settings
        $currentEventcorrelationSettings = $EventcorrelationSettingsTable->getCurrentEventcorrelationSettings();
        $defaultEventcorrelationSettings = $EventcorrelationSettingsTable->getDefaultEventcorrelationSettings();

        $considerStateType = ($defaultEventcorrelationSettings['EVC_CONSIDER_STATETYPE']['value'] & $currentEventcorrelationSettings->get('configuration_option'));
        $showInfoForDisabledService = ($defaultEventcorrelationSettings['EVC_SHOW_INFO_FOR_DISABLED_SERVICES']['value'] & $currentEventcorrelationSettings->get('configuration_option'));
        $stateForDisabledService = $defaultEventcorrelationSettings['EVC_STATUS_DISABLED_SERVICE']['value'];
        if (!empty($currentEventcorrelationSettings)) {
            $stateForDisabledService = $currentEventcorrelationSettings->get('status_disabled_service');
        }
        if (!$HostsTable->existsByIdAndType($id, EVK_HOST)) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        if ($this->request->is('get')) {
            //Ship current EVC for edit

            $evcHosts = $EventcorrelationsTable->getEvcHostsById($id);

            $containerIds = $this->MY_RIGHTS;
            $hosts = [];
            if (!empty($evcHosts)) {
                $hosts = $HostsTable->getHostsByContainerId($containerIds, 'list', 'id', [
                    'Hosts.disabled IN' => [1], //0 is defaut and will get merged so the query is IN(0,1) :facepalming:
                    'Hosts.id IN'       => array_unique(Hash::extract($evcHosts, '{n}.service.host_id'))
                ]);
            }

            try {
                $evcTree = $EventcorrelationsTable->getEvcTreeData($id, array_keys($hosts));
                $rootElement = $EventcorrelationsTable->getRootelementByHostId($id);
            } catch (RecordNotFoundException $e) {
                $evcTree = [];
                $rootElement = null;
            }

            $currentEvcConfigServices = (!empty($rootElement)) ? $EventcorrelationsTable->getCurrentEvcConfig($id) : [];

            if (!empty($evcTree)) {
                $evcTree = Hash::insert($evcTree, '{n}.{*}.{n}.usedBy', []);
                $serviceUsedBy = $EventcorrelationsTable->servicesUsedByList($id, $currentEvcConfigServices);
                if (!empty($serviceUsedBy)) {
                    foreach ($evcTree as $layerIndex => $evcLayerObjects) {
                        if ($layerIndex === 0) {
                            //ignore first layer
                            continue;
                        }
                        foreach ($evcLayerObjects as $evcLayer => $evcLayerObject) {
                            foreach ($evcLayerObject as $evcKey => $evcObject) {
                                $evcTree[$layerIndex][$evcLayer][$evcKey]['usedBy'] = [];
                                if (array_key_exists($evcObject['service_id'], $serviceUsedBy)) {
                                    $evcTree[$layerIndex][$evcLayer][$evcKey]['usedBy'] = $serviceUsedBy[$evcObject['service_id']];
                                }
                            }
                        }
                    }
                }
            }

            //Get current service status
            $serviceUuids = Hash::extract($evcTree, '{n}.{*}.{n}.service.uuid');

            $ServicestatusFields = new ServicestatusFields($this->DbBackend);
            $ServicestatusFields
                ->currentState()
                ->isHardstate()
                ->lastHardState();
            $servicestatus = $ServicestatusTable->byUuids($serviceUuids, $ServicestatusFields);

            $disabledServices = 0;
            foreach ($evcTree as $layerNumber => $layer) {
                foreach ($layer as $parentId => $childNodes) {
                    foreach ($childNodes as $childIndex => $childNode) {
                        $Servicestatus = new Servicestatus([]);
                        if (!empty($servicestatus[$childNode['service']['uuid']])) {
                            $Servicestatus = new Servicestatus(
                                $servicestatus[$childNode['service']['uuid']]['Servicestatus']
                            );
                        }
                        if ($considerStateType && !$Servicestatus->isHardState()) {
                            $Servicestatus->setCurrentState($Servicestatus->getLastHardState());
                        }
                        if ($layerNumber == 0 && $childNode['service']['disabled']) {
                            $Servicestatus->setCurrentState($stateForDisabledService);
                            $Servicestatus->isHardState(true);
                            $Servicestatus->setOutput('DISABLED');
                            $disabledServices++;
                        }
                        $evcTree[$layerNumber][$parentId][$childIndex]['service']['servicestatus'] = $Servicestatus->toArray();
                    }
                }
            }

            //Get EVC Servicetemplates
            $servicetemplates = Api::makeItJavaScriptAble(
                $ServicetemplatesTable->getServicetemplatesByContainerId($this->MY_RIGHTS, 'list', EVK_SERVICE)
            );

            $this->set('evcTree', $evcTree);
            $this->set('rootElement', $rootElement);
            $this->set('stateForDisabledService', $stateForDisabledService);
            $this->set('showInfoForDisabledService', $showInfoForDisabledService);
            $this->set('servicetemplates', $servicetemplates);
            $this->set('disabledServices', $disabledServices);
            $this->viewBuilder()->setOption('serialize', [
                'evcTree',
                'rootElement',
                'servicetemplates',
                'stateForDisabledService',
                'showInfoForDisabledService',
                'disabledServices'
            ]);
            return;
        }


        if ($this->request->is('post') || $this->request->is('put')) {
            $evcTree = $this->request->getData('evcTree', []);
            if (empty($evcTree)) {
                $this->response = $this->response->withStatus(400);
                $this->set('message', [
                    __('Can not save empty event correlation.')
                ]);
                $this->viewBuilder()->setOption('serialize', ['message']);
                return;
            }

            /** @var $HosttemplatesTable HosttemplatesTable */
            $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');
            /** @var $HostsTable HostsTable */
            $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
            /** @var $ServicetemplatesTable ServicetemplatesTable */
            $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');
            /** @var $ServicesTable ServicesTable */
            $ServicesTable = TableRegistry::getTableLocator()->get('Services');
            /** @var  ChangelogsTable $ChangelogsTable */
            $ChangelogsTable = TableRegistry::getTableLocator()->get('Changelogs');

            $User = new User($this->getUser());

            $evcHost = $HostsTable->getHostById($id);

            $EventcorrelationsTable->setTreeScope($id);

            //Save all serviceIds that are currently in use - to detect if virtual services got deleted.
            $oldVirtualServiceIds = $EventcorrelationsTable->getServiceIdsByEvcHostId($id);
            $newVirtualServiceIds = [];

            //Delete old evc tree
            $EventcorrelationsTable->cleanUpEventCorrelation($id);

            $ServicetemplateCache = new KeyValueStore();

            $parentIdsMapping = [];

            krsort($evcTree, SORT_NATURAL);
            foreach ($evcTree as $layerIndex => $services) {
                foreach ($services as $parentIdDontUse => $eventCorrelation) {
                    foreach ($eventCorrelation as $index => $evcService) {
                        if ($EventcorrelationsTable->isNewEvcService($evcService['service']['id'])) {
                            if (!$ServicetemplateCache->has($evcService['service']['servicetemplate_id'])) {
                                $ServicetemplateCache->set(
                                    $evcService['service']['servicetemplate_id'],
                                    $ServicetemplatesTable->getServicetemplateForDiff($evcService['service']['servicetemplate_id'])
                                );
                            }

                            $servicetemplate = $ServicetemplateCache->get($evcService['service']['servicetemplate_id']);

                            //Create new virtual evc services
                            $serviceData = [
                                'Service' => [
                                    'servicetemplate_id' => $evcService['service']['servicetemplate_id'],
                                    'host_id'            => $id,
                                    'name'               => $evcService['service']['servicename'],
                                    'description'        => $evcService['service']['description'],
                                    'disabled'           => 0,
                                ],
                                'Host'    => [
                                    'id'   => $evcHost->get('id'),
                                    'name' => $evcHost->get('name')
                                ]
                            ];
                            $ServiceComparisonForSave = new ServiceComparisonForSave(
                                $serviceData,
                                $servicetemplate,
                                $HostsTable->getContactsAndContactgroupsById($evcHost->get('id')),
                                $HosttemplatesTable->getContactsAndContactgroupsById($evcHost->get('hosttemplate_id'))
                            );

                            $serviceData = $ServiceComparisonForSave->getDataForSaveForAllFields();
                            $serviceData['uuid'] = UUID::v4();
                            $serviceData['service_type'] = EVK_SERVICE;
                            $serviceData['usage_flag'] = EVENTCORRELATION_MODULE;


                            //Add required fields for validation
                            $serviceData['servicetemplate_flap_detection_enabled'] = $servicetemplate['Servicetemplate']['flap_detection_enabled'];
                            $serviceData['servicetemplate_flap_detection_on_ok'] = $servicetemplate['Servicetemplate']['flap_detection_on_ok'];
                            $serviceData['servicetemplate_flap_detection_on_warning'] = $servicetemplate['Servicetemplate']['flap_detection_on_warning'];
                            $serviceData['servicetemplate_flap_detection_on_critical'] = $servicetemplate['Servicetemplate']['flap_detection_on_critical'];
                            $serviceData['servicetemplate_flap_detection_on_unknown'] = $servicetemplate['Servicetemplate']['flap_detection_on_unknown'];

                            $service = $ServicesTable->newEntity($serviceData);
                            $ServicesTable->save($service);

                            $extDataForChangelog = $ServicesTable->resolveDataForChangelog(['Service' => $serviceData]);
                            $changelog_data = $ChangelogsTable->parseDataForChangelog(
                                'add',
                                'services',
                                $service->get('id'),
                                OBJECT_SERVICE,
                                $evcHost->get('container_id'),
                                $User->getId(),
                                $evcHost->get('name') . '/' . $evcService['service']['servicename'],
                                array_merge(['Service' => $serviceData], $extDataForChangelog)
                            );

                            if ($changelog_data) {
                                /** @var Changelog $changelogEntry */
                                $changelogEntry = $ChangelogsTable->newEntity($changelog_data);
                                $ChangelogsTable->save($changelogEntry);
                            }

                        } else {
                            if ($layerIndex === 0) {
                                //Update existing "real" layer 0 service
                                $service = $ServicesTable->get($evcService['service']['id']);
                                $ServicesTable->setUsageFlagById($evcService['service']['id'], EVENTCORRELATION_MODULE);
                            } else {
                                //Update existing virtual service
                                $service = $ServicesTable->get($evcService['service']['id']);
                                $service->set('name', $evcService['service']['servicename']);
                                $service->set('servicetemplate_id', $evcService['service']['servicetemplate_id']);

                                if ($service->get('usage_flag') & EVENTCORRELATION_MODULE) {
                                    //Service already has the flag for given module
                                } else {
                                    $newFlag = $service->get('usage_flag') + EVENTCORRELATION_MODULE;
                                    $service->set('usage_flag', $newFlag);
                                }
                                $ServicesTable->save($service);
                            }
                        }

                        if ($layerIndex > 0) {
                            $newVirtualServiceIds[] = $service->get('id');
                        }

                        //Save evcTree
                        $evcNode = $EventcorrelationsTable->newEntity([
                            'parent_id'  => $parentIdsMapping[$evcService['parent_id']] ?? null, //Last layer has no parent = ROOT_ELEMENT
                            'host_id'    => $id,
                            'service_id' => $service->get('id'),
                            'operator'   => $evcService['operator']
                        ]);

                        $EventcorrelationsTable->save($evcNode);

                        // Map old or uu-id- parent ids to new DB auto increment parent ids.
                        $parentIdsMapping[$evcService['id']] = $evcNode->get('id');
                    }
                }
            }

            $virtualServiceIdsToDelete = array_diff($oldVirtualServiceIds, $newVirtualServiceIds);
            if (!empty($virtualServiceIdsToDelete)) {
                foreach ($virtualServiceIdsToDelete as $virtualServiceIdToDelete) {
                    $serviceEntity = $ServicesTable->get($virtualServiceIdToDelete);
                    $ServicesTable->delete($serviceEntity);
                }
            }
        }

        $this->set('success', true);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }

    /**
     * @param int|null $id
     * @deprecated
     */
    public function delete($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */

        if (!$HostsTable->existsByIdAndType($id, EVK_HOST)) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        /** @var \App\Model\Entity\Host $host */
        $host = $HostsTable
            ->find()
            ->contain([
                'HostescalationsHostMemberships',
                'HostdependenciesHostMemberships',
                'HostsToContainersSharing'
            ])
            ->where([
                'Hosts.id' => $id
            ])
            ->firstOrFail();

        if (!$this->allowedByContainerId($host->getContainerIds())) {
            $this->render403();
            return;
        }

        $allVirtualServicesByEvcHostId = $EventcorrelationsTable->getServiceIdsByEvcHostId($id);
        $firstLayerServices = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($id);
        $firstLayerServicesIds = array_unique(Hash::extract($firstLayerServices, '{n}.service.id'));

        foreach ($allVirtualServicesByEvcHostId as $virtualServiceId) {
            if ($EventcorrelationsTable->isServiceUsedByOtherEvcs($virtualServiceId, $id) === true) {
                $this->response = $this->response->withStatus(400);
                $usedBy = [
                    'EVENTCORRELATION_MODULE' => [
                        'baseUrl' => '#',
                        'state'   => 'EventcorrelationsHostUsedBy',
                        'message' => __('Used by Eventcorrelation module'),
                        'module'  => 'EventcorrelationModule',
                        'id'      => $id
                    ]
                ];
                $this->set('success', false);
                $this->set('id', $id);
                $this->set('usedBy', $usedBy); //Because used by another event correlations
                $this->viewBuilder()->setOption('serialize', ['success', 'usedBy', 'id']);
                return;
            }
        }

        $eventCorrelation = $EventcorrelationsTable->find()
            ->where([
                'Eventcorrelations.host_id' => $id
            ])
            ->whereNull([
                'Eventcorrelations.parent_id'
            ])
            ->first();

        if ($eventCorrelation === null) {
            //This is an EVK which is empty (has no tree in the database)
            $host = $HostsTable->get($id);
            $User = new User($this->getUser());

            //Delete the host
            if ($HostsTable->__delete($host, $User)) {
                $this->set('success', true);
                $this->set('message', __('Host successfully deleted'));
                $this->viewBuilder()->setOption('serialize', ['success']);
                return;
            }
            return;
        }
        if ($EventcorrelationsTable->deleteAll(['Eventcorellations.host_id' => $id])) {
            //Update usage flag for first layer services
            foreach ($firstLayerServicesIds as $firstLayerServicesId) {
                if ($EventcorrelationsTable->isServiceUsedByOtherEvcs($firstLayerServicesId, $id) === false) {
                    $ServicesTable->removeUsageFlagById($firstLayerServicesId, EVENTCORRELATION_MODULE);
                }
            }

            $services = $ServicesTable->getServicesByHostIdForDelete($id, true);

            //Host + Services of the host are not in use by any Modules an can be deleted.
            $User = new User($this->getUser());

            //Delete services of the host
            /** @var \App\Model\Entity\Service $serviceEntity */
            foreach ($services as $serviceEntity) {
                $service = $ServicesTable->find()
                    ->contain([
                        'ServiceescalationsServiceMemberships',
                        'ServicedependenciesServiceMemberships'
                    ])
                    ->where([
                        'Services.id' => $serviceEntity->get('id')
                    ])
                    ->firstOrFail();

                $ServicesTable->__delete($service, $User);
            }

            $host = $HostsTable->get($id);

            //Delete the host
            if ($HostsTable->__delete($host, $User)) {
                $this->set('success', true);
                $this->set('message', __('Host successfully deleted'));
                $this->viewBuilder()->setOption('serialize', ['success']);
                return;
            }

            $this->set('success', false);
            $this->set('message', __('Error while deleting host'));
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }
        $this->response = $this->response->withStatus(400);
        $this->set('success', false);
        $this->set('message', __('Could not delete event correlation'));
        $this->viewBuilder()->setOption('serialize', ['success', 'message']);
    }

    /**
     * @param int|null $hostId
     */
    public function usedBy($hostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship template
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');

        if (!$HostsTable->existsByIdAndType($hostId, EVK_HOST)) {
            throw new NotFoundException(__('Invalid event correlation'));
        }

        $allEvcHosts = $HostsTable->getHostsByTypeId(EVK_HOST, $this->MY_RIGHTS);

        $allEvcHosts = Hash::combine($allEvcHosts, '{n}.id', '{n}');
        $allEvcHosts = Hash::insert($allEvcHosts, '{n}.usedBy', []);

        $additionalDetails = $EventcorrelationsTable->inUse(array_keys($allEvcHosts));
        foreach ($additionalDetails as $additionalDetail) {
            $HostConditions = new HostConditions([
                'Hosts.host_type' => EVK_HOST
            ]);
            if ($this->hasRootPrivileges === false) {
                $HostConditions->setContainerIds($this->MY_RIGHTS);
            }

            //ITC-1822
            $hasViewPermission = $this->hasRootPrivileges;
            $hasWritePermission = $this->hasRootPrivileges;

            if ($this->hasRootPrivileges === false) {
                $firstLayerHosts = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($additionalDetail['EvcHosts']['id']);
                $hostIds = array_unique(Hash::extract($firstLayerHosts, '{n}.service.host_id'));

                if (!empty($hostIds)) {
                    //EVC has own services defined
                    //Check each first layer service (through host) container permissions

                    $allContainerIdsInEvc = [];
                    foreach ($hostIds as $hostId) {
                        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
                        $allContainerIdsInEvc[$hostId] = $host->getContainerIds();
                    }

                    $EvcContainerPermissions = new EvcContainerPermissions(
                        $this->MY_RIGHTS_LEVEL,
                        $allContainerIdsInEvc,
                        $additionalDetail['EvcHosts']['container_id']
                    );
                    $hasWritePermission = $EvcContainerPermissions->hasEditPermission();
                    $hasViewPermission = $EvcContainerPermissions->hasViewPermission();

                } else {
                    //Empty EVC (no own services defined)
                    //Use EVC host container_id itself for permission check
                    $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$additionalDetail['EvcHosts']['container_id']]);
                    $hasWritePermission = $ContainerPermissions->hasPermission();
                    $hasViewPermission = $ContainerPermissions->hasViewPermission();
                }
            }

            $additionalDetail['EvcHosts']['hasViewPermission'] = $hasViewPermission;
            $additionalDetail['EvcHosts']['hasWritePermission'] = $hasWritePermission;

            $allEvcHosts[$additionalDetail['id']]['usedBy'][$additionalDetail['Eventcorrelations']['host_id']] = $additionalDetail;
        }
        $usedBy = $allEvcHosts[$hostId];

        $this->set('usedBy', $usedBy);
        $this->viewBuilder()->setOption('serialize', ['usedBy']);
    }

    /**
     * @param int|null $serviceId
     */
    public function serviceUsedBy($serviceId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */

        if (!$ServicesTable->existsById($serviceId)) {
            throw new NotFoundException(__('Service not found'));
        }
        /** @var \App\Model\Entity\Service $service */
        $service = $ServicesTable->getServiceByIdForPermissionsCheck($serviceId);
        if (!$this->allowedByContainerId($service->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $eventcorrelations = $EventcorrelationsTable->getAllEvcsByServiceIds($serviceId);
        foreach ($eventcorrelations as $index => $eventcorrelation) {
            $hasViewPermission = $this->hasRootPrivileges;
            $hasWritePermission = $this->hasRootPrivileges;

            if ($this->hasRootPrivileges === false) {
                $firstLayerHosts = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($eventcorrelation['Hosts']['id']);
                $hostIds = array_unique(Hash::extract($firstLayerHosts, '{n}.service.host_id'));

                if (!empty($hostIds)) {
                    //EVC has own services defined
                    //Check each first layer service (through host) container permissions

                    $allContainerIdsInEvc = [];
                    foreach ($hostIds as $hostId) {
                        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
                        $allContainerIdsInEvc[$hostId] = $host->getContainerIds();
                    }

                    $EvcContainerPermissions = new EvcContainerPermissions(
                        $this->MY_RIGHTS_LEVEL,
                        $allContainerIdsInEvc,
                        $eventcorrelation['Hosts']['container_id']
                    );
                    $hasWritePermission = $EvcContainerPermissions->hasEditPermission();
                    $hasViewPermission = $EvcContainerPermissions->hasViewPermission();
                } else {
                    //Empty EVC (no own services defined)
                    //Use EVC host container_id itself for permission check
                    $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$eventcorrelation['Hosts']['container_id']]);
                    $hasWritePermission = $ContainerPermissions->hasPermission();
                    $hasViewPermission = $ContainerPermissions->hasViewPermission();
                }

            }

            $eventcorrelations[$index]['hasViewPermission'] = $hasViewPermission;
            $eventcorrelations[$index]['hasWritePermission'] = $hasWritePermission;
        }

        $this->set('service', $service);
        $this->set('eventcorrelations', $eventcorrelations);
        $this->viewBuilder()->setOption('serialize', ['service', 'eventcorrelations']);
    }

    /**
     * @param int|null $hostId
     */
    public function hostUsedBy($hostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var EventcorrelationsTable $EventcorrelationsTable */
        $EventcorrelationsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.Eventcorrelations');
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */

        if (!$HostsTable->existsById($hostId)) {
            throw new NotFoundException(__('Host not found'));
        }

        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);

        if (!$this->allowedByContainerId($host->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $serviceIds = $EventcorrelationsTable->getAllServiceIdsByHostIdAndEvcUsageFlag($hostId);

        $eventcorrelations = $EventcorrelationsTable->getAllEvcsByServiceIds($serviceIds);
        foreach ($eventcorrelations as $index => $eventcorrelation) {
            $hasViewPermission = $this->hasRootPrivileges;
            $hasWritePermission = $this->hasRootPrivileges;

            if ($this->hasRootPrivileges === false) {
                $firstLayerHosts = $EventcorrelationsTable->getFirstLayerServicesByEvcHostId($eventcorrelation['Hosts']['id']);
                $hostIds = array_unique(Hash::extract($firstLayerHosts, '{n}.service.host_id'));

                if (!empty($hostIds)) {
                    //EVC has own services defined
                    //Check each first layer service (through host) container permissions

                    $allContainerIdsInEvc = [];
                    foreach ($hostIds as $hostId) {
                        $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
                        $allContainerIdsInEvc[$hostId] = $host->getContainerIds();
                    }

                    $EvcContainerPermissions = new EvcContainerPermissions(
                        $this->MY_RIGHTS_LEVEL,
                        $allContainerIdsInEvc,
                        $eventcorrelation['Hosts']['container_id']
                    );
                    $hasWritePermission = $EvcContainerPermissions->hasEditPermission();
                    $hasViewPermission = $EvcContainerPermissions->hasViewPermission();
                } else {
                    //Empty EVC (no own services defined)
                    //Use EVC host container_id itself for permission check
                    $ContainerPermissions = new ContainerPermissions($this->MY_RIGHTS_LEVEL, [$eventcorrelation['Hosts']['container_id']]);
                    $hasWritePermission = $ContainerPermissions->hasPermission();
                    $hasViewPermission = $ContainerPermissions->hasViewPermission();
                }

            }

            $eventcorrelations[$index]['hasViewPermission'] = $hasViewPermission;
            $eventcorrelations[$index]['hasWritePermission'] = $hasWritePermission;
        }
        $this->set('host', $host);
        $this->set('eventcorrelations', $eventcorrelations);
        $this->viewBuilder()->setOption('serialize', ['host', 'eventcorrelations']);
    }

    public function summary_view($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship template
            return;
        }

        if ($this->request->is('get')) {
            /** @var $HostsTable HostsTable */
            $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

            $evcHost = $HostsTable->getHostById($id);
            $this->set('evcHost', $evcHost);
            $this->viewBuilder()->setOption('serialize', ['evcHost']);
        }
    }


    /**
     * @param int|null $id
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     */
    public function summary_view_table($id = null) {
        //Fetch data and pass to frontend/view
        $this->view($id, true);

        $User = new User($this->getUser());
        $UserTime = UserTime::fromUser($User);

        if ($this->isHtmlRequest() === true) {
            //Ship table as html
            return;
        }

        if ($this->isHtmlRequest() === false) {
            //Ship table as PDF
            /** @var $HostsTable HostsTable */
            $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
            $evcHost = $HostsTable->getHostById($id);

            $this->set('evcHost', $evcHost);
            $this->set('UserTime', $UserTime);

            $this->viewBuilder()->setOption(
                'pdfConfig',
                [
                    'download' => true,
                    'filename' => $evcHost->get('name') . '_' . date('dmY_his') . '.pdf',
                ]
            );
        }
    }

    /**
     * @param int $id
     * @deprecated
     * @todo ------------------------MEEEEEEEEEEEEEEEEEEEEEEEEEEE----------------
     */
    public function getServiceWithStateById($id) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }
        if (!$this->Service->exists($id)) {
            throw new NotFoundException('Invalid Service');
        }
        $eventcorrelation_settings = $this->EventcorrelationSetting->currentEventcorrelationSettings();
        $default_eventcorrelation_settings = $this->EventcorrelationSetting->defaultEventcorrelationSettings();
        $considerStateType = ($default_eventcorrelation_settings['EVC_CONSIDER_STATETYPE']['value'] & $eventcorrelation_settings['EventcorrelationSetting']['configuration_option']);

        $tempService = $this->Service->find('first', [
            'recursive'  => -1,
            'contain'    => [
                'Servicetemplate' => [
                    'fields' => [
                        'Servicetemplate.id',
                        'Servicetemplate.name'
                    ],
                ],
                'Host'            => [
                    'fields' => [
                        'Host.id',
                        'Host.name',
                        'Host.uuid'
                    ],
                ],
            ],
            'fields'     => [
                'Service.id',
                'Service.disabled',
                'Service.name',
                'Service.uuid',
                'Service.service_type'
            ],
            'conditions' => [
                'Service.id' => $id,
            ],
        ]);

        /** @var $ServicestatusTable ServicestatusTableInterface */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->wildcard();
        $serviceState = $ServicestatusTable->byUuid($tempService['Service']['uuid'], $ServicestatusFields);

        $currentState = null;
        if (!empty($serviceState)) {
            $Servicestatus = new \itnovum\openITCOCKPIT\Core\Servicestatus($serviceState['Servicestatus']);
            $currentState = $Servicestatus->currentState();
            if ($considerStateType && !$Servicestatus->isHardState()) {
                $currentState = $Servicestatus->getLastHardState();
            }
        }
        $serviceName = $tempService['Service']['name'];
        if (empty($serviceName)) {
            $serviceName = $tempService['Servicetemplate']['name'];
        }

        $service[$tempService['Service']['id']] = [
            'hostId'          => $tempService['Host']['id'],
            'hostName'        => $tempService['Host']['name'],
            'hostUuid'        => $tempService['Host']['uuid'],
            'serviceId'       => $tempService['Service']['id'],
            'serviceDisabled' => $tempService['Service']['disabled'],
            'serviceName'     => $serviceName,
            'currentState'    => $currentState,
            'serviceUuid'     => $tempService['Service']['uuid'],
            'serviceType'     => $tempService['Service']['service_type'],
            'Servicetemplate' => [
                'id'   => $tempService['Servicetemplate']['id'],
                'name' => $tempService['Servicetemplate']['name'],
            ],
        ];

        $this->set('service', $service);
        $this->set('_serialize', ['service']);
    }

    public function loadServicesByString() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        if ($this->request->is('POST')) {
            $selected = $this->request->getData('selected', []);
            $notServiceId = $this->request->getData('notServiceId', []);
        } else {
            $selected = $this->request->getQuery('selected', []);
            $notServiceId = $this->request->getQuery('notServiceId', []);
        }

        $ServiceFilter = new ServiceFilter($this->request);


        $ServiceCondition = new ServiceConditions($ServiceFilter->indexFilter());
        $ServiceCondition->setNotConditions([
            'Services.host_id' => $notServiceId
        ]);
        $ServiceCondition->setIncludeDisabled(true);
        $ServiceCondition->setContainerIds($this->MY_RIGHTS);

        /** @var $ServicesTable ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');

        $services = Api::makeItJavaScriptAble(
            $ServicesTable->getServicesForAngular($ServiceCondition, $selected)
        );


        $this->set('services', $services);
        $this->viewBuilder()->setOption('serialize', ['services']);
    }

    public function evcWidget() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }

        /** @var WidgetsTable $WidgetsTable */
        $WidgetsTable = TableRegistry::getTableLocator()->get('Widgets');
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if ($this->request->is('get')) {
            $widgetId = (int)$this->request->getQuery('widgetId', 0);
            if (!$WidgetsTable->existsById($widgetId)) {
                throw new \RuntimeException('Invalid widget id');
            }

            $widget = $WidgetsTable->getWidgetByIdAsCake2($widgetId);

            //Check host permissions
            $host = null;
            if (is_numeric($widget['Widget']['host_id'])) {
                $host = $HostsTable->getHostByIdForPermissionCheck($widget['Widget']['host_id']);
            }

            $hostId = null;
            if (!empty($host)) {
                $hostId = (int)$widget['Widget']['host_id'];
                if ($this->hasRootPrivileges === false) {
                    if (!$this->allowedByContainerId($host->getContainerIds(), false)) {
                        $hostId = null;
                    }
                }
            }

            $acl = [
                'evc' => [
                    'view' => isset($this->PERMISSIONS['eventcorrelationmodule']['eventcorrelations']['view']),
                ]
            ];

            $this->set('host_id', $hostId);
            $this->set('ACL', $acl);

            $this->viewBuilder()->setOption('serialize', ['host_id', 'ACL']);
            return;
        }


        if ($this->request->is('post')) {
            $hostId = (int)$this->request->getData('host_id', 0);

            if ($hostId === 0) {
                $hostId = null;
            }

            $widgetId = (int)$this->request->getData('Widget.id', 0);

            if (!$WidgetsTable->existsById($widgetId)) {
                throw new \RuntimeException('Invalid widget id');
            }

            $widget = $WidgetsTable->get($widgetId);
            $widget->set('host_id', $hostId);

            $WidgetsTable->save($widget);
            if ($widget->hasErrors()) {
                $this->serializeCake4ErrorMessage($widget);
                return;
            } else {
                $this->set('host_id', $hostId);
                $this->viewBuilder()->setOption('serialize', ['host_id']);
                return;
            }
        }
        throw new MethodNotAllowedException();
    }

    /****************************
     *       AJAX METHODS       *
     ****************************/


    /**
     * @param int|null $id
     * @throws \App\Lib\Exceptions\MissingDbBackendException
     * @throws \Exception
     */
    public function getServicestatus($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template
            return;
        }
        $User = new User($this->getUser());
        $UserTime = $User->getUserTime();

        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();
        /** @var HoststatusTableInterface $HoststatusTable */
        $HoststatusTable = $this->DbBackend->getHoststatusTable();

        if (!$ServicesTable->existsById($id)) {
            throw new NotFoundException(__('Invalid service'));
        }


        $service = $ServicesTable->getServiceById($id);// ->getServiceByIdForPermissionsCheck($id);
        if (!$this->allowedByContainerId($service->getContainerIds(), false)) {
            $this->render403();
            return;
        }

        $servicename = (!$service->get('name')) ? $service->get('servicetemplate')->get('name') : $service->get('name');
        $Host = new Host($service->get('host'));
        $Service = new Service($service, $servicename);

        $HoststatusFields = new HoststatusFields($this->DbBackend);
        $HoststatusFields->wildcard();

        $hoststatus = $HoststatusTable->byUuid($Host->getUuid(), $HoststatusFields);
        if (empty($hoststatus)) {
            $Hoststatus = new Hoststatus([], $UserTime);
        } else {
            $Hoststatus = new Hoststatus($hoststatus['Hoststatus'], $UserTime);
        }

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields->wildcard();

        $servicestatus = $ServicestatusTable->byUuid($Service->getUuid(), $ServicestatusFields);
        if (empty($servicestatus)) {
            $Servicestatus = new Servicestatus([], $UserTime);
        } else {
            $Servicestatus = new Servicestatus($servicestatus['Servicestatus'], $UserTime);
        }
        $this->set('host', $Host->toArray());
        $this->set('service', $Service->toArray());
        $this->set('servicestatus', $Servicestatus->toArrayForBrowser());
        $this->set('hoststatus', $Hoststatus->toArrayForBrowser());
        $this->viewBuilder()->setOption('serialize', ['host', 'service', 'servicestatus', 'hoststatus']);
    }

    /**
     * @throws \Exception
     */
    public function loadContainers() {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        /** @var ContainersTable $ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');

        if ($this->hasRootPrivileges === true) {
            $containers = $ContainersTable->easyPath($this->MY_RIGHTS, OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        } else {
            $containers = $ContainersTable->easyPath($this->getWriteContainers(), OBJECT_HOST, [], $this->hasRootPrivileges, [CT_HOSTGROUP]);
        }

        $this->set('containers', Api::makeItJavaScriptAble($containers));
        $this->viewBuilder()->setOption('serialize', ['containers']);
    }

    /**
     * @param $containerId
     * @param int $hostId
     * @throws \Exception
     */
    public function loadElementsByContainerId($containerId, $hostId = 0) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        $hostId = (int)$hostId;
        $hosttemplateType = EVK_HOSTTEMPLATE;

        /** @var $ContainersTable ContainersTable */
        $ContainersTable = TableRegistry::getTableLocator()->get('Containers');
        /** @var $ContactsTable ContactsTable */
        $ContactsTable = TableRegistry::getTableLocator()->get('Contacts');
        /** @var $TimeperiodsTable TimeperiodsTable */
        $TimeperiodsTable = TableRegistry::getTableLocator()->get('Timeperiods');
        /** @var $ContactgroupsTable ContactgroupsTable */
        $ContactgroupsTable = TableRegistry::getTableLocator()->get('Contactgroups');
        /** @var $HosttemplatesTable HosttemplatesTable */
        $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');
        /** @var $HostsTable HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if (!$ContainersTable->existsById($containerId)) {
            throw new NotFoundException(__('Invalid container'));
        }

        if ($hostId != 0) {
            try {
                $host = $HostsTable->get($hostId);
                $hosttemplateType = $host->get('host_type');
            } catch (RecordNotFoundException $e) {
                //Ignore error
            }
        }

        $containerIds = $ContainersTable->resolveChildrenOfContainerIds($containerId);

        $hosttemplates = $HosttemplatesTable->getHosttemplatesByContainerId($containerIds, 'list', $hosttemplateType);
        $hosttemplates = Api::makeItJavaScriptAble($hosttemplates);

        $timeperiods = $TimeperiodsTable->timeperiodsByContainerId($containerIds, 'list');
        $timeperiods = Api::makeItJavaScriptAble($timeperiods);
        $checkperiods = $timeperiods;

        $contacts = $ContactsTable->contactsByContainerId($containerIds, 'list');
        $contacts = Api::makeItJavaScriptAble($contacts);

        $contactgroups = $ContactgroupsTable->getContactgroupsByContainerId($containerIds, 'list', 'id');
        $contactgroups = Api::makeItJavaScriptAble($contactgroups);

        $this->set('hosttemplates', $hosttemplates);
        $this->set('timeperiods', $timeperiods);
        $this->set('checkperiods', $checkperiods);
        $this->set('contacts', $contacts);
        $this->set('contactgroups', $contactgroups);

        $this->viewBuilder()->setOption('serialize', [
            'hosttemplates',
            'timeperiods',
            'checkperiods',
            'contacts',
            'contactgroups'
        ]);
    }

    /**
     * @param int $hosttemplateId
     */
    public function loadHosttemplate($hosttemplateId) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        /** @var HosttemplatesTable $HosttemplatesTable */
        $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');

        if (!$HosttemplatesTable->existsById($hosttemplateId)) {
            throw new NotFoundException(__('Invalid host template'));
        }

        $hosttemplate = $HosttemplatesTable->getHosttemplateForEdit($hosttemplateId);


        $this->set('hosttemplate', $hosttemplate);
        $this->viewBuilder()->setOption('serialize', ['hosttemplate']);
    }

    public function validateModalAddVServices() {
        $EventcorrelationForm = new EventcorrelationForm();
        $data = $this->request->getData(null, []);
        $EventcorrelationForm->execute($data);

        if (!empty($EventcorrelationForm->getErrors())) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $EventcorrelationForm->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        $evcId = (int)$this->request->getData('current_evc.id', 0);

        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var ServicestatusTableInterface $ServicestatusTable */
        $ServicestatusTable = $this->DbBackend->getServicestatusTable();
        /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */
        $EventcorrelationSettingsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.EventcorrelationSettings');

        if (!$HostsTable->existsByIdAndType($evcId, EVK_HOST)) {
            throw new NotFoundException(__('Invalid event correlation host'));
        }

        $evcHost = $HostsTable->getHostById($evcId);

        //Event correlation settings
        $currentEventcorrelationSettings = $EventcorrelationSettingsTable->getCurrentEventcorrelationSettings();
        $defaultEventcorrelationSettings = $EventcorrelationSettingsTable->getDefaultEventcorrelationSettings();

        $considerStateType = ($defaultEventcorrelationSettings['EVC_CONSIDER_STATETYPE']['value'] & $currentEventcorrelationSettings->get('configuration_option'));


        // get current service status for the first layer (real services)
        $layerIndex = (int)$this->request->getData('current_evc.layerIndex', -1);
        $services = [];

        // $serviceIds can be real mysql IDs or random UUIDs because of the service does not exists jet (new vServices)
        $serviceIds = $data['service_ids'];

        $ServicestatusFields = new ServicestatusFields($this->DbBackend);
        $ServicestatusFields
            ->currentState()
            ->isHardstate()
            ->lastHardState();

        //Remove fake uuid "service ids"
        $numericServiceIds = array_filter($serviceIds, 'is_numeric');
        $servicestatusArray = $ServicestatusTable->byUuids(
            $ServicesTable->getServiceUuidsByServiceIds($numericServiceIds),
            $ServicestatusFields
        );
        $parentUUID = 'ui-id-' . UUID::v4();

        $count = 1;
        foreach ($serviceIds as $serviceId) {
            if ($layerIndex === 0) {
                $serviceEntity = null;
                if (is_numeric($serviceId)) {
                    $serviceEntity = $ServicesTable->getServiceById($serviceId);
                }

                $Servicestatus = new Servicestatus([]);
                if (isset($servicestatusArray[$serviceEntity->get('uuid')])) {
                    $Servicestatus = new Servicestatus($servicestatusArray[$serviceEntity->get('uuid')]['Servicestatus']);
                }

                if ($considerStateType && !$Servicestatus->isHardState()) {
                    $Servicestatus->setCurrentState($Servicestatus->getLastHardState());
                }

                // Services layer 0 (first row on the left side)
                $services[] = [
                    'id'         => $parentUUID . '_' . $count,
                    'parent_id'  => $parentUUID,
                    'host_id'    => $evcId,
                    'service_id' => $serviceId,
                    'operator'   => null,
                    'service'    => [
                        'id'                 => $serviceId,
                        'servicetemplate_id' => $serviceEntity->get('servicetemplate_id'),
                        'host_id'            => $serviceEntity->get('host_id'),
                        'name'               => $serviceEntity->get('name'),
                        'description'        => $serviceEntity->get('description'),
                        'service_type'       => $serviceEntity->get('service_type'),
                        'uuid'               => $serviceEntity->get('uuid'),
                        'disabled'           => (int)$serviceEntity->get('disabled'),
                        'host'               => [
                            'id'   => $serviceEntity->get('host')->get('id'),
                            'name' => $serviceEntity->get('host')->get('name'),
                        ],
                        'servicename'        => $serviceEntity->get('name') ?? $serviceEntity->get('servicetemplate')->get('name'),
                        'servicestatus'      => $Servicestatus->toArray()
                    ]
                ];
                $count++;
            }
        }


        // add virtual services
        $updates = [];
        if ($layerIndex === 0) {
            $updates[$layerIndex][$parentUUID] = $services;
        }

        $updates[$layerIndex + 1][$parentUUID . '_vService'] = [ // . '_vService'
            [
                'id'         => $parentUUID,
                'parent_id'  => null,
                'host_id'    => $evcId,
                'service_id' => $parentUUID . '_vService',
                'operator'   => $data['operator'] . ($data['operator_modifier'] > 0 ? $data['operator_modifier'] : ''),
                'usedBy'     => [], //New services can't be used by any other EVC
                'service'    => [
                    'id'                 => $parentUUID . '_vService',
                    'servicetemplate_id' => $data['servicetemplate_id'],
                    'host_id'            => $evcId,
                    'name'               => $data['servicename'],
                    'description'        => null,
                    'service_type'       => EVK_SERVICE,
                    'uuid'               => null,
                    'disabled'           => 0,
                    'host'               => [
                        'id'   => $evcId,
                        'name' => $evcHost->get('name'),
                    ],
                    'servicename'        => $data['servicename'],

                    //No services have no status
                    'servicestatus'      => [
                        'cssClass' => 'newVirtualServiceStatusColor'
                    ]
                ]
            ]
        ];


        $this->set('success', true);
        $this->set('updates', $updates);
        $this->viewBuilder()
            ->setOption('serialize', ['success', 'updates'])
            ->setOption('jsonOptions', JSON_FORCE_OBJECT); //Force keys 0, 1 etc to get always a object {} in JSON and never an array []
    }

    public function validateModalEditVServices() {
        $EventcorrelationForm = new EventcorrelationForm();
        $data = $this->request->getData(null, []);
        $EventcorrelationForm->execute($data);

        if (!empty($EventcorrelationForm->getErrors())) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $EventcorrelationForm->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        $evcId = (int)$this->request->getData('current_evc.id', 0);
        $parentId = $this->request->getData('current_evc.evc_node_id', null);
        $layerIndex = (int)$this->request->getData('current_evc.layerIndex', -1);

        if ($parentId === null) {
            throw new BadRequestException('current_evc.evc_node_id is missing in request!');
        }
        if ($layerIndex === -1) {
            throw new BadRequestException('current_evc.layerIndex is missing in request!');
        }

        if ($layerIndex === 1) {
            //Load "real" services for the first layer

            /** @var HostsTable $HostsTable */
            $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
            /** @var ServicesTable $ServicesTable */
            $ServicesTable = TableRegistry::getTableLocator()->get('Services');
            /** @var ServicestatusTableInterface $ServicestatusTable */
            $ServicestatusTable = $this->DbBackend->getServicestatusTable();
            /** @var EventcorrelationSettingsTable $EventcorrelationSettingsTable */
            $EventcorrelationSettingsTable = TableRegistry::getTableLocator()->get('EventcorrelationModule.EventcorrelationSettings');

            if (!$HostsTable->existsByIdAndType($evcId, EVK_HOST)) {
                throw new NotFoundException(__('Invalid event correlation host'));
            }

            //Event correlation settings
            $currentEventcorrelationSettings = $EventcorrelationSettingsTable->getCurrentEventcorrelationSettings();
            $defaultEventcorrelationSettings = $EventcorrelationSettingsTable->getDefaultEventcorrelationSettings();

            $considerStateType = ($defaultEventcorrelationSettings['EVC_CONSIDER_STATETYPE']['value'] & $currentEventcorrelationSettings->get('configuration_option'));


            // get current service status for the first layer (real services)
            $services = [];
            $serviceIds = $data['service_ids'];

            $ServicestatusFields = new ServicestatusFields($this->DbBackend);
            $ServicestatusFields
                ->currentState()
                ->isHardstate()
                ->lastHardState();

            $servicestatusArray = $ServicestatusTable->byUuids(
                $ServicesTable->getServiceUuidsByServiceIds($serviceIds),
                $ServicestatusFields
            );

            $count = 1;
            foreach ($serviceIds as $serviceId) {
                $serviceEntity = null;
                if (is_numeric($serviceId)) {
                    $serviceEntity = $ServicesTable->getServiceById($serviceId);
                }

                $Servicestatus = new Servicestatus([]);
                if (isset($servicestatusArray[$serviceEntity->get('uuid')])) {
                    $Servicestatus = new Servicestatus($servicestatusArray[$serviceEntity->get('uuid')]['Servicestatus']);
                }

                if ($considerStateType && !$Servicestatus->isHardState()) {
                    $Servicestatus->setCurrentState($Servicestatus->getLastHardState());
                }

                // Services layer 0 (first row on the left side)
                $services[] = [
                    'id'         => $parentId . '_' . $count,
                    'parent_id'  => $parentId,
                    'host_id'    => $evcId,
                    'service_id' => $serviceId,
                    'operator'   => null,
                    'service'    => [
                        'id'                 => $serviceId,
                        'servicetemplate_id' => $serviceEntity->get('servicetemplate_id'),
                        'host_id'            => $serviceEntity->get('host_id'),
                        'name'               => $serviceEntity->get('name'),
                        'description'        => $serviceEntity->get('description'),
                        'service_type'       => $serviceEntity->get('service_type'),
                        'uuid'               => $serviceEntity->get('uuid'),
                        'disabled'           => (int)$serviceEntity->get('disabled'),
                        'host'               => [
                            'id'   => $serviceEntity->get('host')->get('id'),
                            'name' => $serviceEntity->get('host')->get('name'),
                        ],
                        'servicename'        => $serviceEntity->get('name') ?? $serviceEntity->get('servicetemplate')->get('name'),
                        'servicestatus'      => $Servicestatus->toArray()
                    ]
                ];
                $count++;
            }

            $this->set('success', true);
            $this->set('services', $services);
            $this->viewBuilder()
                ->setOption('serialize', ['success', 'services']);
            return;

        }

        //Layer > 1
        $this->set('success', true);
        $this->viewBuilder()
            ->setOption('serialize', ['success']);
    }

}
