<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
namespace EventcorrelationModule;


class EvcTableRenderer {

    /**
     * @var array
     */
    private $evcTree = [];

    /**
     * @var null|string
     */
    private $uuidForServiceAlert = null;

    /**
     * @var bool
     */
    private $useEmoji = false;

    /**
     * @var bool
     */
    private $useEvcServicestatusDirective = false;

    /**
     * EvcTableRenderer constructor.
     * @param array $evcTree
     * @param null|string $uuidForServiceAlert
     */
    public function __construct(array $evcTree, $uuidForServiceAlert = null, $useEmoji = false, $useEvcServicestatusDirective = false) {
        $this->evcTree = $evcTree;
        $this->uuidForServiceAlert = $uuidForServiceAlert;
        $this->useEmoji = $useEmoji;
        $this->useEvcServicestatusDirective = $useEvcServicestatusDirective;
    }

    /**
     * @param string|null $uuidForServiceAlert
     */
    public function setUuidForServiceAlert(?string $uuidForServiceAlert): void {
        $this->uuidForServiceAlert = $uuidForServiceAlert;
    }

    /**
     * @param bool $useEmoji
     */
    public function setUseEmoji(bool $useEmoji): void {
        $this->useEmoji = $useEmoji;
    }

    /**
     * @param bool $useEvcServicestatusDirective
     */
    public function setUseEvcServicestatusDirective(bool $useEvcServicestatusDirective): void {
        $this->useEvcServicestatusDirective = $useEvcServicestatusDirective;
    }


    /**
     * @return string
     */
    public function getEvcDataAsTable(): string {
        $reFormatData = $this->reformatData();
        if (!$reFormatData) {
            $html = '<span class="text-danger"><i class="fa fa-exclamation-triangle" aria-hidden="true"></i> Invalid configuration</span>';
            if ($this->uuidForServiceAlert) {
                $html = 'Invalid configuration';
            }
            return $html;
        }
        $htmlTableStructureArray = $this->prepareDataForHtmlCreate($reFormatData);
        $html = $this->createHmtl($htmlTableStructureArray);
        return $html;
    }

    public function reformatData() {
        $evcTree = $this->evcTree;
        $tmpParentId = null;
        $tableSettings = [];
        $subCounter = 1;
        $sizeOfLayers = sizeof($evcTree);
        foreach ($evcTree as $layerNumber => $layerData) {
            $tmpLayerNumber = $layerNumber + 1;
            if ($tmpLayerNumber < $sizeOfLayers) {
                $tableSettings[$tmpLayerNumber] = [];
            }
            foreach ($layerData as $parentId => $evcNodeData) {
                foreach ($evcNodeData as $key => $data) {
                    if (empty($data['service']['host']['name'])) {
                        // config invalid
                        return false;
                    }

                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['serviceId'] = $data['service']['id'];
                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['hostName'] = $data['service']['host']['name'];
                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['serviceName'] = $data['service']['servicename'];
                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['uuid'] = $data['service']['uuid'];
                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['operator'] = $data['operator'];
                    if ($data['service']['disabled'] === 1) {
                        $tableSettings[$tmpLayerNumber - 1][$data['id']]['current_state'] = '-2';
                    } else if ($data['service']['disabled'] === 0 && $data['service']['servicestatus']['currentState'] >= 0) {
                        $tableSettings[$tmpLayerNumber - 1][$data['id']]['current_state'] = $data['service']['servicestatus']['currentState'];
                    } else {
                        $tableSettings[$tmpLayerNumber - 1][$data['id']]['current_state'] = '-2';
                    }

                    $tableSettings[$tmpLayerNumber - 1][$data['id']]['disabled'] = $data['service']['disabled'];
                    if (!isset($tableSettings[$tmpLayerNumber - 1][$data['id']]['serviceCounter']) && $layerNumber === 0 && $tmpLayerNumber < $sizeOfLayers) {
                        $tableSettings[$tmpLayerNumber - 1][$data['id']]['serviceCounter'] = 1;
                    }
                    if (!isset($tableSettings[$tmpLayerNumber][$parentId]) && $tmpLayerNumber < $sizeOfLayers) {
                        $tableSettings[$tmpLayerNumber][$parentId]['serviceCounter'] = 0;
                    }
                    if ($tmpParentId !== $parentId) {
                        $tmpParentId = $parentId;
                        $subCounter = 1;
                    } else {
                        $subCounter++;
                    }
                    if ($layerNumber > 0 && !empty($tableSettings[($tmpLayerNumber - 1)][$data['id']]) && $tmpLayerNumber < $sizeOfLayers) {
                        $tableSettings[$tmpLayerNumber][$tmpParentId]['serviceCounter'] += $tableSettings[$tmpLayerNumber - 1][$data['id']]['serviceCounter'];
                    }

                }
                if ($layerNumber === 0) {
                    $tableSettings[$tmpLayerNumber][$parentId] = [
                        'serviceCounter' => $subCounter
                    ];
                }
            }
        }
        ksort($tableSettings);
        return $tableSettings;
    }

    /**
     * @param array $filteredData
     * @return array
     */
    public function prepareDataForHtmlCreate($filteredData): array {
        $htmlTableStructure = [];
        foreach ($filteredData as $layerNumber => $layerDataArray) {
            $rowCounter = 0;
            foreach ($layerDataArray as $id => $layerData) {
                if ($layerNumber === 0) {
                    $htmlTableStructure[$rowCounter][$layerNumber] = $layerData;
                    $rowCounter++;
                }
                if ($layerNumber > 0) {
                    $htmlTableStructure[$rowCounter][$layerNumber] = $layerData;
                    $rowCounter++;
                    for ($i = 1; $i < $layerData['serviceCounter']; $i++) {
                        $rowCounter++;
                    }
                }
            }
        }
        return $htmlTableStructure;
    }

    /**
     * @param array $htmlTableStructureArray
     * @return string
     */
    public function createHmtl($htmlTableStructureArray): string {
        $html = '';
        $statusMapping = [
            'ok',
            'warning',
            'critical',
            'unknown'
        ];

        if (!empty($htmlTableStructureArray)) {
            $html = '<table id="evcSummary">';
            foreach ($htmlTableStructureArray as $htmlTableStructure) {
                $html .= '<tr>';
                foreach ($htmlTableStructure as $key => $column) {
                    $column['current_state'] = is_null($column['current_state'])?'-1':$column['current_state'];
                    $html .= '<td ';
                    if ($column['serviceCounter'] > 1) {
                        $html .= ' rowspan="' . $column['serviceCounter'] . '"';
                    }
                    if ($key === 0 && $column['current_state'] >= 0) {
                        $html .= sprintf(
                            ' class="border%sEvcSummary" ',
                            ucfirst($statusMapping[$column['current_state']])
                        );
                    } else if ($key === 0 && $column['current_state'] == '-1') {
                        $html .= ' class="borderNotMonitoredEvcSummary" ';
                    } else if ($key === 0 && $column['current_state'] == '-2') {
                        $html .= ' class="borderDisabledEvcSummary disabledEvcSummary" ';
                    } else {
                        if ($key > 0 && !array_key_exists($column['current_state'], $statusMapping)) {
                            $html .= ' class="notMonitoredEvcSummary no-bordered" ';
                        } else {
                            $html .= sprintf(
                                ' class="%sEvcSummary no-bordered" ',
                                $statusMapping[$column['current_state']]
                            );
                        }
                    }

                    if($this->useEvcServicestatusDirective === true){
                        $html .= sprintf('ng-mouseenter="showEvcServicestatusDelayed(%s)"', $column['serviceId']);
                        $html .= 'ng-mouseleave="cancelEvcServicestatusDelayTimer()"';
                    }

                    $html .= '>';
                    if ($key === 0) {
                        $html .= sprintf(
                            '<span class="firstLayerServiceSpan" title="%s/%s">%s/%s</span>',
                            h($column['hostName']),
                            h($column['serviceName']),
                            h($column['hostName']),
                            h($column['serviceName'])

                        );
                    } else {
                        $alertMarker = '';
                        if ($this->uuidForServiceAlert === $column['uuid'] && $column['current_state'] > 0) {
                            $alertMarker = '&#9888;';
                            if ($this->useEmoji) {
                                $alertMarker = '⚠️';
                            }
                        } else if ($this->uuidForServiceAlert === $column['uuid'] && $column['current_state'] == 0) {
                            $alertMarker = '&#9432;';
                            if ($this->useEmoji) {
                                $alertMarker = '✅️';
                            }
                        }
                        $html .= sprintf(
                            '<div class="gatter txt-color-white">%s</div><span class="txt-color-white virtualLayerService">%s %s</span>',
                            strtoupper($column['operator']),
                            h($column['serviceName']),
                            $alertMarker
                        );
                        h($column['serviceName']);
                    }
                    $html .= '</td>';
                }
                $html .= '</tr>';
            }
            $html .= '</table>';
        }
        return $html;
    }

}