<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace EventcorrelationModule\Form;

use Cake\Form\Form;
use Cake\Form\Schema;
use Cake\Validation\Validator;

/**
 * Class EventcorrelationForm
 * @package EventcorrelationModule\Form
 */
class EventcorrelationForm extends Form {
    protected function _buildSchema(Schema $schema): Schema {
        return $schema
            ->addField('servicename', ['type' => 'string'])
            ->addField('servicetemplate_id', ['type' => 'int'])
            ->addField('service_ids', ['type' => 'array'])
            ->addField('operator', ['type' => 'string'])
            ->addField('operator_modifier', ['type' => 'int']);
    }

    public function validationDefault(Validator $validator): Validator {
        $validator
            ->allowEmptyString('servicename', null, false)
            ->requirePresence('servicename', 'create');


        $validator
            ->integer('servicetemplate_id')
            ->allowEmptyString('servicetemplate_id', null, false)
            ->requirePresence('servicetemplate_id', 'create')
            ->greaterThan('servicetemplate_id', 0);

        $validator
            ->isArray('service_ids')
            ->allowEmptyArray('service_ids', null, false)
            ->requirePresence('service_ids', 'create');

        $validator
            ->allowEmptyString('operator', null, false)
            ->requirePresence('operator', 'create')
            ->inList('operator', ['and', 'or', 'eq', 'min'], __('Operator out of bounds'))
            ->add('operator', 'and_or', [
                'rule'    => function ($value, $context) {
                    if (isset($context['data']['service_ids']) && in_array($context['data']['operator'], ['and', 'or'], true)) {
                        return count($context['data']['service_ids']) >= 2;
                    }

                    //No "and" or "or" operator
                    return true;
                },
                'message' => __('This operator requires at least two Services')
            ])
            ->add('operator', 'eq', [
                'rule'    => function ($value, $context) {
                    if (isset($context['data']['service_ids']) && $context['data']['operator'] === 'eq') {
                        return count($context['data']['service_ids']) === 1;
                    }

                    //No "eq" operator
                    return true;
                },
                'message' => __('The EQUAL Operator can only have one Service')
            ])
            ->add('operator', 'min', [
                'rule'    => function ($value, $context) {
                    if (isset($context['data']['service_ids']) && isset($context['data']['operator_modifier']) && $context['data']['operator'] === 'min') {
                        return count($context['data']['service_ids']) >= $context['data']['operator_modifier'];
                    }

                    //No "eq" operator
                    return true;
                },
                'message' => __('The MIN Operator requires a Modifier which is in the range of 1 - X (where X is the number of the selected services)')
            ]);


        $validator
            ->integer('operator_modifier')
            ->greaterThan('operator_modifier', 0, __('Operator modifier has to be greater than 0.'), function ($context) {
                if (isset($context['data']['operator']) && $context['data']['operator'] === 'min') {
                    //Min operator requires a number
                    return true;
                }

                return false;
            });

        return $validator;
    }
}