<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace EventcorrelationModule\Model\Table;

use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * EventcorrelationSettings Model
 *
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting get($primaryKey, $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting newEntity($data = null, array $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting[] newEntities(array $data, array $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting|bool save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting[] patchEntities($entities, array $data, array $options = [])
 * @method \EventcorrelationModule\Model\Entity\EventcorrelationSetting findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin \Cake\ORM\Behavior\TimestampBehavior
 */
class EventcorrelationSettingsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('eventcorrelation_settings');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('id')
            ->allowEmptyString('id', 'create');

        $validator
            ->scalar('configuration_option')
            ->maxLength('configuration_option', 255)
            ->range('configuration_option', [0, 15], __('Out of range (0-15)'))
            ->requirePresence('configuration_option', 'create')
            ->notEmptyString('configuration_option');

        $validator
            ->scalar('monitoring_system')
            ->maxLength('monitoring_system', 255)
            ->inList('monitoring_system', ['naemon', 'nagios', 'statusengine'], __('Supported options are "naemon", "nagios" or "statusengine"'))
            ->requirePresence('monitoring_system', 'create')
            ->notEmptyString('monitoring_system');

        return $validator;
    }

    /**
     * @return array|\Cake\Datasource\EntityInterface|\EventcorrelationModule\Model\Entity\EventcorrelationSetting|null
     */
    public function getCurrentEventcorrelationSettings() {
        $record = $this->find()->first();

        if($record === null){
            $record = $this->newEntity([
                'monitoring_system' => 'statusengine',
                'configuration_option' => '0',
                'status_disabled_service' => 3 //unknown as default status for disabled service in evc
            ]);
        }

        return $record;
    }

    /**
     * @return array
     */
    public function getDefaultEventcorrelationSettings() {
        $all_eventcorrelation_settings = [
            'EVC_CONSIDER_STATETYPE'   => [
                'value' => 1 << 0,
                'info'  => __('Consider service status type (0 => every state type, 1 => only hard state)'),
            ],
            'EVC_CONSIDER_STATE_COUNT' => [
                'value' => 1 << 1,
                'info'  => __('Consider number of states'),
            ],
            'EVC_REFLECT_CRITICAL_STATE' => [
                'value' => 1 << 2,
                'info'  => __('If enabled, the event correlation host will go in "down" state if the last (root) event correlation service is "critical"'),
            ],
            'EVC_MONITORING_SYSTEM'    => [
                'value' => 'naemon',
                'info'  => __('Monitoring System'),
            ],
            'EVC_STATUS_DISABLED_SERVICE'    => [
                'value' => 3, //service state unknown
                'info'  => __('Use this status as status for disabled services in an event correlation'),
            ],
            'EVC_SHOW_INFO_FOR_DISABLED_SERVICES' => [
                'value' => 1 << 3,
                'info'  => __('If enabled, shows note if event correlation contains disabled services'),
            ],
        ];

        return $all_eventcorrelation_settings;
    }
}
