angular.module('openITCOCKPIT')
    .controller('EventcorrelationsEditCorrelationController', function($scope, $http, $stateParams, $timeout, $state, NotyService, RedirectService){

        /*** Notes ***/
        // Only services where parent_id === null have the suffix _vService in the key of the JSON tree

        $scope.id = $stateParams.id;
        $scope.highlightHostId = $stateParams.highlightHostId;
        $scope.highlightServiceId = $stateParams.highlightServiceId;

        $scope.init = true;
        $scope.showSpinner = false;

        /** private methods **/
        $scope.loadServicetemplates = function(){
            $http.get("/eventcorrelation_module/eventcorrelations/editCorrelation/" + $scope.id + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.evcTree = result.data.evcTree;
                $scope.rootElement = result.data.rootElement;
                $scope.init = false;

            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };

        /**
         * array_unique — Removes duplicate values from an array
         *
         * @param arr
         * @returns {*[]}
         */
        var array_unique = function(arr){
            // is array?
            if(typeof arr.filter !== "function"){
                return arr;
            }

            var cb = function(value, index, self){
                return self.indexOf(value) === index;
            };

            return arr.filter(cb);
        }

        /**
         * Get all services as object by layerIndex
         * @param layerIndex
         * @returns array with objects
         */
        var getServicesByLayerIndex = function(layerIndex){
            var layerServices = [];
            layerIndex = String(layerIndex);

            for(var evcLayer in $scope.evcTree){
                if(evcLayer === '$$hashKey' || evcLayer !== layerIndex){
                    //$$hashKey is used by AngularJS to track changes
                    continue;
                }

                for(var services in $scope.evcTree[evcLayer]){
                    if(services === '$$hashKey'){
                        //$$hashKey is used by AngularJS to track changes
                        continue;
                    }

                    for(var eventCorrelation in $scope.evcTree[evcLayer][services]){
                        if(eventCorrelation === '$$hashKey'){
                            //$$hashKey is used by AngularJS to track changes
                            continue;
                        }

                        var evc = $scope.evcTree[evcLayer][services][eventCorrelation];

                        layerServices.push(evc);
                    }
                }
            }

            return layerServices;
        };

        /**
         * @param layerIndex
         * @returns {[]}
         */
        var getServicesByLayerIndexForSelect = function(layerIndex, currentEvcId){
            var servicesInLayer = getServicesByLayerIndex(layerIndex);
            var services = [];

            for(var index in servicesInLayer){
                var evcIdAsString = String(servicesInLayer[index].id);

                //Add _vService suffix if missing (vServices loaded from database don't have the _vService suffix)
                if(!evcIdAsString.endsWith('_vService')){
                    evcIdAsString = evcIdAsString + '_vService';
                }

                var vServiceInUse = servicesInLayer[index].parent_id !== null; //used by add modal

                if(typeof currentEvcId !== "undefined"){
                    if(servicesInLayer[index].parent_id === currentEvcId){ //used by edit modal
                        vServiceInUse = false;
                    }
                }

                services.push({
                    key: evcIdAsString,
                    value: {
                        Service: {
                            id: evcIdAsString,
                            name: servicesInLayer[index].service.name,
                            servicename: servicesInLayer[index].service.servicename
                        },
                        Host: {
                            id: servicesInLayer[index].service.host.id,
                            name: servicesInLayer[index].service.host.name
                        },
                    },
                    vServiceInUse: vServiceInUse
                });
            }

            return services;
        };

        /**
         * Get all service_id's as array by layerIndex
         * @param layerIndex
         * @returns array
         */
        var getServicesIdsByLayerIndex = function(layerIndex){
            var layerServices = getServicesByLayerIndex(layerIndex);
            var layerServicesIds = [];

            for(var index in layerServices){
                layerServicesIds.push(layerServices[index].service_id);
            }

            return layerServicesIds;
        };

        /**
         * @param layerIndex
         * @param parentId
         * @returns {[]}
         */
        var getServicesIdsByLayerIndexAndParentId = function(layerIndex, parentId){
            var layerServices = getServicesByLayerIndex(layerIndex);
            var layerServicesIds = [];

            for(var index in layerServices){
                if(layerServices[index].parent_id === parentId){
                    if(layerIndex === 0){
                        layerServicesIds.push(layerServices[index].service_id);
                    }else{
                        layerServicesIds.push(layerServices[index].id + '_vService');
                    }
                }
            }

            return layerServicesIds;
        };

        /**
         * Remove one vService from the evcTree and also remove the parent vService if no more vServices
         * where attached to the parent vService
         *
         * @param int layerIndex
         * @param object evcNode
         */
        var deleteEvcNode = function(layerIndex, evcNodeId){
            //debugger
            // Remove AngularJs $$hashKey crap
            $scope.evcTree = JSON.parse(angular.toJson($scope.evcTree));


            var currentParentId = null;
            var currentEvcNode = null;
            // find the parent id in json
            find_parent_loop:
                for(var parentEvcId in $scope.evcTree[layerIndex]){
                    for(var k in $scope.evcTree[layerIndex][parentEvcId]){

                        //Find the evcNode by the given id
                        if($scope.evcTree[layerIndex][parentEvcId][k].id === evcNodeId){
                            // parent exists
                            currentParentId = $scope.evcTree[layerIndex][parentEvcId][k].parent_id;
                            currentEvcNode = $scope.evcTree[layerIndex][parentEvcId][k];
                            break find_parent_loop;
                        }
                    }
                    if(currentParentId !== null){
                        //leave loop because parent exists
                        break;
                    }
                }

            //delete services from 1st level in json
            if(layerIndex === 1){
                delete $scope.evcTree[0][evcNodeId];
                if(_.isEmpty($scope.evcTree[0])){

                    $scope.evcTree = [];

                    //Remove old lines
                    $('.connector').remove();
                }
            }

            if(_.size($scope.evcTree[layerIndex][parentEvcId]) === 1){
                // delete layer with number if only single element exists(data type object {})
                delete $scope.evcTree[layerIndex][parentEvcId];

                //Remove layer and all PARENT layers to the left, because this layer is empty now!
                if(_.isEmpty($scope.evcTree[layerIndex])){

                    var amountOfLayersToDelete = $scope.evcTree.length - layerIndex;
                    $scope.evcTree.splice(layerIndex, amountOfLayersToDelete);

                    //Cleanup done - no parents to delete anymore
                    return;
                }

            }else{
                //remove element with id if layer consists more items than one (data type array [])
                $scope.evcTree[layerIndex][parentEvcId].splice(k, 1);

                //return, clean up done, all affected elements has been removed
                return;
            }

            //Delete vServices to the right
            if(currentParentId !== null){
                deleteEvcNode(layerIndex + 1, currentParentId);
            }
        };

        var checkForMissingParents = function(){
            // Remove AngularJs $$hashKey crap
            $scope.evcTree = JSON.parse(angular.toJson($scope.evcTree));

            for(var layerIndex in $scope.evcTree){
                for(var parentEvcId in $scope.evcTree[layerIndex]){

                    //Check if this parent exists
                    var parentExists = false;
                    var evcLayerToCheck = parseInt(layerIndex, 10) + 1;
                    if(typeof $scope.evcTree[evcLayerToCheck] != 'undefined'){
                        for(var parentEvcIdToCheck in $scope.evcTree[evcLayerToCheck]){
                            for(var k in $scope.evcTree[evcLayerToCheck][parentEvcIdToCheck]){
                                if(String($scope.evcTree[evcLayerToCheck][parentEvcIdToCheck][k].id) === String(parentEvcId)){
                                    parentExists = true;
                                    break;
                                }
                            }
                            if(parentExists === true){
                                break;
                            }
                        }
                    }
                    if(parentExists === false){
                        var container = $scope.evcTree[layerIndex][parentEvcId];
                        //resolve original event correlation for processing
                        delete $scope.evcTree[layerIndex][parentEvcId];
                        for(var evcService in container){
                            container[evcService].parent_id = null;
                            // for example: Layer 1{1} with oldKey => 60 [2 Elements with parent_id = 60]
                            // after delete node with id 60
                            // Layer 1{2} with keys =>  61_vService[1] and 66_vService[1] and parent_id = null
                            var key = container[evcService].id + '_vService';
                            $scope.evcTree[layerIndex][key] = [];
                            $scope.evcTree[layerIndex][key].push(container[evcService]);
                        }
                    }
                }
            }
        };

        /**
         * @param operatorString
         * @returns {{modifier: *, operator: string}|{modifier: number, operator: *}}
         */
        var getParsedOperator = function(operatorString){
            var first3Chars = operatorString.substring(0, 3);

            switch(first3Chars){
                case 'min':
                    //min1, min 10, min300
                    return {
                        operator: 'min',
                        modifier: parseInt(operatorString.substring(3), 10)
                    };

                default:
                    //AND, OR, EQ
                    return {
                        operator: operatorString,
                        modifier: 0
                    };
            }
        };

        var getOperatorString = function(modalvService){
            switch(modalvService.operator){
                case 'min':
                    //min1, min 10, min300
                    return modalvService.operator + String(modalvService.operator_modifier);

                default:
                    //AND, OR, EQ
                    return modalvService.operator;
            }
        };

        /**
         *
         * @returns {boolean}
         */
        var validateEvcTreeBeforeSave = function(){
            $scope.layerWithErrors = {};
            $scope.evcNodeWithErrors = {};

            // Remove AngularJs $$hashKey crap
            var evcTree = JSON.parse(angular.toJson($scope.evcTree));

            var lastLayerIndex = evcTree.length - 1;
            lastLayerIndex = String(lastLayerIndex);

            for(var layerIndex in evcTree){
                var layerHasErrors = false;
                for(var services in evcTree[layerIndex]){
                    for(var eventCorrelation in evcTree[layerIndex][services]){
                        var evcNode = evcTree[layerIndex][services][eventCorrelation];

                        //Check all layers for a parent service - except the last layer
                        if(evcNode.parent_id === null && layerIndex !== lastLayerIndex){
                            layerHasErrors = true;
                            $scope.evcNodeWithErrors[evcNode.id] = true;
                        }

                        //Check the last layer - only allow one element with parent_id = null!
                        if(layerIndex === lastLayerIndex){
                            if(Object.keys(evcTree[layerIndex]).length > 1){
                                layerHasErrors = true;
                                $scope.evcNodeWithErrors[evcNode.id] = true;
                            }
                        }
                    }
                }
                if(layerHasErrors === true){
                    $scope.layerWithErrors[layerIndex] = true;
                }
            }

            if(Object.keys($scope.layerWithErrors).length > 0){
                return false;
            }

            if(Object.keys($scope.evcNodeWithErrors).length > 0){
                return false;
            }

            return true;
        };

        /** public methods **/
        $scope.load = function(){
            $http.get("/eventcorrelation_module/eventcorrelations/editCorrelation/" + $scope.id + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.evcTree = result.data.evcTree;
                $scope.rootElement = result.data.rootElement;
                $scope.servicetemplates = result.data.servicetemplates;
                $scope.stateForDisabledService = parseInt(result.data.stateForDisabledService, 10);
                $scope.showInfoForDisabledService = parseInt(result.data.showInfoForDisabledService, 10);
                $scope.disabledServices = parseInt(result.data.disabledServices, 10);
                $scope.init = false;

            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };

        $scope.positionEvcTimeout = function(){
            $timeout($scope.renderEvc, 100);
        };

        $scope.renderEvc = function(){
            //console.log(JSON.stringify($scope.evcTree));
            $EvcRenderer = new EvcRenderer();
            $EvcRenderer.renderEvc($scope.evcTree);
        };

        $scope.showAddVServiceModal = function(layerIndex){
            $scope.modalCurrentLayerIndex = layerIndex;
            $('#evcServicesModal').modal('show');

            $scope.modalVService = {
                servicename: '',
                servicetemplate_id: 0,
                service_ids: [],
                operator: null,
                operator_modifier: 0,
                current_evc: {
                    id: $scope.id,
                    layerIndex: layerIndex,
                    mode: 'add'
                }
            };

            if(layerIndex === 0){
                //Load "real" services for the first layer
                //Use "real" service ids from the database (services.id)
                $scope.loadServices('', getServicesIdsByLayerIndex(0));
            }

            if(layerIndex > 0){
                //Load "virtual" services for current layer
                //All other layers use the evcId_vService as key in the select box => 5_vService
                $scope.servicesForSelect = [];
                $scope.servicesForSelect = getServicesByLayerIndexForSelect(layerIndex);
            }
        };

        $scope.showEditVServiceModal = function(layerIndex, eventCorrelation){
            $scope.modalCurrentLayerIndex = layerIndex - 1;

            var layerIndexToLoadServicesFrom = layerIndex - 1;
            $('#evcServicesModal').modal('show');

            var operator = getParsedOperator(eventCorrelation.operator);
            $scope.modalVService = {
                servicename: eventCorrelation.service.name,
                servicetemplate_id: eventCorrelation.service.servicetemplate_id,
                service_ids: getServicesIdsByLayerIndexAndParentId(layerIndexToLoadServicesFrom, eventCorrelation.id),
                operator: operator.operator,
                operator_modifier: operator.modifier,
                current_evc: {
                    id: $scope.id,
                    layerIndex: layerIndex,
                    mode: 'edit',
                    evc_node_id: eventCorrelation.id,
                    old_service_ids: getServicesIdsByLayerIndexAndParentId(layerIndexToLoadServicesFrom, eventCorrelation.id)
                }
            };

            if(layerIndexToLoadServicesFrom === 0){
                //Load "real" services for the first layer
                //Use "real" service ids from the database (services.id)
                $scope.loadServices('', getServicesIdsByLayerIndex(0));
            }

            if(layerIndexToLoadServicesFrom > 0){
                //Load "virtual" services for current layer
                //All other layers use the evcId_vService as key in the select box => 5_vService
                $scope.servicesForSelect = [];
                $scope.servicesForSelect = getServicesByLayerIndexForSelect(layerIndexToLoadServicesFrom, eventCorrelation.id);
            }
        };

        $scope.loadServices = function(searchString, selected){
            if(typeof selected === "undefined"){
                selected = [];
            }

            //Add current selected services of the select box also to selected list
            if(typeof $scope.modalVService.service_ids !== "undefined"){
                for(var index in $scope.modalVService.service_ids){
                    selected.push($scope.modalVService.service_ids[index]);
                }
            }

            selected = array_unique(selected);

            var params = {
                'angular': true,
                // Old GET filters - only here for reference. Delete this block if the year is 2023
                //'filter[servicename]': searchString,
                //'selected[]': selected,
                //'notServiceId': $scope.id
            };

            // ITC-2712 Change load function to use POST
            var data = {
                filter: {
                    servicename: searchString
                },
                selected: selected,
                notServiceId: $scope.id
            }

            $http.post("/eventcorrelation_module/eventcorrelations/loadServicesByString.json", data, {
                params: params
            }).then(function(result){
                $scope.servicesForSelect = result.data.services;
            });
        };

        $scope.validateModalForm = function(){
            if($scope.modalVService.current_evc.mode === 'add'){
                $scope.validateModalAddVServices();
            }else{
                $scope.validateModalEditVServices();
            }
        };

        // add new services
        $scope.validateModalAddVServices = function(){
            $scope.showSpinner = true;
            $http.post("/eventcorrelation_module/eventcorrelations/validateModalAddVServices.json?angular=true",
                $scope.modalVService
            ).then(function(result){
                $scope.errors = {};

                var updates = result.data.updates;

                // Remove AngularJs $$hashKey crap
                $scope.evcTree = JSON.parse(angular.toJson($scope.evcTree));

                for(var layerIndexToUpdate in updates){
                    var layerIndexToUpdateInt = parseInt(layerIndexToUpdate, 10);

                    if(typeof $scope.evcTree[layerIndexToUpdateInt] === "undefined"){
                        //Create new layer in evcTree
                        $scope.evcTree[layerIndexToUpdateInt] = {};
                    }

                    if(layerIndexToUpdate === "0"){ //this is an json object {} so we get strings !!!
                        //Add new 1st layer services
                        for(var newParentId in updates[layerIndexToUpdate]){

                            //Add new 1st layer services to evcTree as array
                            $scope.evcTree[layerIndexToUpdateInt][newParentId] = [];

                            for(var jsonKey in updates[layerIndexToUpdate][newParentId]){
                                $scope.evcTree[layerIndexToUpdateInt][newParentId].push(updates[layerIndexToUpdate][newParentId][jsonKey]);
                            }
                        }
                    }

                    //Add new vServices (this can always be only one service !)
                    if(layerIndexToUpdateInt > 0){

                        for(var newParentIdvService in updates[layerIndexToUpdate]){

                            //Add new vService to evcTree as array
                            $scope.evcTree[layerIndexToUpdateInt][newParentIdvService] = [];

                            //The new vService has always the key "0" !
                            updates[layerIndexToUpdate][newParentIdvService]["0"].usedBy = []; //This needs to be an array but PHP's JSON_FORCE_OBJECT force this into an {} object
                            $scope.evcTree[layerIndexToUpdateInt][newParentIdvService].push(updates[layerIndexToUpdate][newParentIdvService]["0"]);

                            var newParentIdWithoutvServiceSuffix = updates[layerIndexToUpdate][newParentIdvService]["0"].id;

                            //Update old vServices with the new parentId (NO _vService prefix!)
                            //Only update if we do not add any 1st layer services.
                            //If we have any 1st layer services, the service handles all that for us.
                            if(updates.hasOwnProperty("0") === false){

                                //Create new evc "container" with new parent id as key (no _vService suffix)
                                $scope.evcTree[$scope.modalVService.current_evc.layerIndex][newParentIdWithoutvServiceSuffix] = [];

                                for(var index in $scope.modalVService.service_ids){

                                    //Get old parentLess node
                                    var oldKeyToUpdate = $scope.modalVService.service_ids[index];
                                    var evcNodeToUpdateArray = $scope.evcTree[$scope.modalVService.current_evc.layerIndex][oldKeyToUpdate];

                                    //Remove old and parent less evcNode
                                    delete $scope.evcTree[$scope.modalVService.current_evc.layerIndex][oldKeyToUpdate];

                                    for(var evcIndex in evcNodeToUpdateArray){
                                        var evcNodeToUpdate = evcNodeToUpdateArray[evcIndex];
                                        evcNodeToUpdate.parent_id = newParentIdWithoutvServiceSuffix;

                                        $scope.evcTree[$scope.modalVService.current_evc.layerIndex][newParentIdWithoutvServiceSuffix].push(evcNodeToUpdate);
                                    }
                                }
                            }
                        }
                    }
                }

                setTimeout(function(){
                    //Avoid that the user can see the "jumping" evc rerender so obviously
                    $('#evcServicesModal').modal('hide');
                    $scope.showSpinner = false;
                }, 150);
            }, function errorCallback(result){
                $scope.showSpinner = false;
                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;
                }
            });
        };

        //Edit vServices
        $scope.validateModalEditVServices = function(){
            $scope.showSpinner = true;
            $http.post("/eventcorrelation_module/eventcorrelations/validateModalEditVServices.json?angular=true",
                $scope.modalVService
            ).then(function(result){
                $scope.errors = {};

                // Remove AngularJs $$hashKey crap
                $scope.evcTree = JSON.parse(angular.toJson($scope.evcTree));

                //Update parent service (operator string, servicetemplate_id, service name)
                evcNode = null;

                find_evc_node:
                    for(var services in $scope.evcTree[$scope.modalVService.current_evc.layerIndex]){
                        for(var eventCorrelation in $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services]){
                            evcNode = $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services][eventCorrelation];

                            //Find the vService node that was edited
                            if(evcNode.id === $scope.modalVService.current_evc.evc_node_id){
                                $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services][eventCorrelation].operator = getOperatorString($scope.modalVService);
                                $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services][eventCorrelation].service.servicename = $scope.modalVService.servicename;
                                $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services][eventCorrelation].service.name = $scope.modalVService.servicename;
                                $scope.evcTree[$scope.modalVService.current_evc.layerIndex][services][eventCorrelation].service.servicetemplate_id = $scope.modalVService.servicetemplate_id;

                                break find_evc_node;
                            }
                        }
                    }

                if(evcNode === null){
                    NotyService.genericError({
                        message: 'Error: Could not find parent node!'
                    });
                    return;
                }

                if($scope.modalVService.current_evc.layerIndex === 1){
                    //Server returned all 1st layer services - simply replace the old services in the json with the new one
                    //layerIndexToUpdate = 0;
                    if(typeof $scope.evcTree[0][evcNode.id] !== "undefined"){
                        $scope.evcTree[0][evcNode.id] = result.data.services;
                    }
                }

                if($scope.modalVService.current_evc.layerIndex > 1){
                    var servicesToAdd = _.difference($scope.modalVService.service_ids, $scope.modalVService.current_evc.old_service_ids);
                    var servicesToDelete = _.difference($scope.modalVService.current_evc.old_service_ids, $scope.modalVService.service_ids);
                    var previousLayerIndex = $scope.modalVService.current_evc.layerIndex - 1;

                    if(!_.isEmpty(servicesToAdd)){
                        for(var i in servicesToAdd){
                            // servicesToAdd can only contain vServices where parent_id === null
                            // so the jsonKey has an _vService suffix.

                            var jsonKey = servicesToAdd[i];
                            var evcNodeToAdd = $scope.evcTree[previousLayerIndex][jsonKey][0];

                            //Delete old node
                            delete $scope.evcTree[previousLayerIndex][jsonKey];

                            //Move evd node to new parent
                            evcNodeToAdd.parent_id = evcNode.id;

                            $scope.evcTree[previousLayerIndex][evcNode.id].push(evcNodeToAdd);
                        }
                    }

                    if(!_.isEmpty(servicesToDelete)){
                        var evcsToKeep = [];
                        for(var index in $scope.evcTree[previousLayerIndex][evcNode.id]){
                            // The value of the select box has the _vService suffix!

                            var idToCheck = $scope.evcTree[previousLayerIndex][evcNode.id][index].id + '_vService';
                            if(!in_array(idToCheck, servicesToDelete)){
                                //Keep this service in current evc node
                                evcsToKeep.push($scope.evcTree[previousLayerIndex][evcNode.id][index]);
                            }else{
                                //This service got removed from current evc node
                                //Set parent_id to null and create new _vService record in json.

                                var evcServiceToMove = $scope.evcTree[previousLayerIndex][evcNode.id][index];

                                var vServiceJsonKey = evcServiceToMove.id + '_vService';
                                evcServiceToMove.parent_id = null;

                                $scope.evcTree[previousLayerIndex][vServiceJsonKey] = [
                                    evcServiceToMove
                                ];
                            }
                        }

                        $scope.evcTree[previousLayerIndex][evcNode.id] = evcsToKeep;
                    }
                }

                setTimeout(function(){
                    //Avoid that the user can see the "jumping" evc rerender so obviously
                    $('#evcServicesModal').modal('hide');
                    $scope.showSpinner = false;
                }, 150);
            }, function errorCallback(result){
                $scope.showSpinner = false;
                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;
                }
            });
        };

        $scope.deleteEvcNodeClick = function(layerIndex, evcNode){
            deleteEvcNode(layerIndex, evcNode.id);
            checkForMissingParents();
        };

        $scope.saveEvcClick = function(){
            var result = validateEvcTreeBeforeSave();

            if(result === false){
                NotyService.genericError();
                return;
            }

            $http.post("/eventcorrelation_module/eventcorrelations/editCorrelation/" + $scope.id + ".json",
                {
                    evcTree: $scope.evcTree,
                    evcId: $scope.id
                }
            ).then(function(result){
                var url = $state.href('EventcorrelationsEditCorrelation', {id: $scope.id});
                NotyService.genericSuccess({
                    message: '<u><a href="' + url + '" class="txt-color-white"> '
                        + $scope.successMessage.objectName
                        + '</a></u> ' + $scope.successMessage.message
                });

                $state.go('EventcorrelationsIndex').then(function(){
                    NotyService.scrollTop();
                });
            }, function errorCallback(result){
                NotyService.genericError();
                console.error(result);
            });

        };

        //Fire on page load
        $scope.load();
    });
